# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2020-05-19 00:08
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('adyengo', '0021_auto_20180912_1526'),
    ]

    operations = [
        migrations.AlterField(
            model_name='recurringpaymentresult',
            name='psp_reference',
            field=models.CharField(blank=True, max_length=150, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-05-27 22:22
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('adyengo', '0009_auto_20160528_0015'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='payment_method',
            field=models.CharField(blank=True, choices=[('paypal', 'PayPal'), ('bankTransfer_DE', 'German Banktransfer'), ('card', 'All debit and credit cards'), ('visa', 'Visa'), ('bankTransfer', 'All banktransfers'), ('elv', 'ELV'), ('amex', 'Amex'), ('mc', 'Master Card'), ('directdebit_NL', 'Direct Debit (Netherlands)'), ('ideal', 'iDEAL'), ('directEbanking', 'SofortUberweisung'), ('bankTransfer_NL', 'Dutch Banktransfer')], max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='recurringpaymentresult',
            name='result_code',
            field=models.CharField(choices=[('Authorised', 'Authorised'), ('Refused', 'Refused'), ('Error', 'Error')], max_length=30),
        ),
        migrations.AlterField(
            model_name='session',
            name='country_code',
            field=models.CharField(blank=True, choices=[('DE', 'Germany'), ('NL', 'Netherlands'), ('BE', 'Belgium'), ('GB', 'United Kingdom')], max_length=2),
        ),
        migrations.AlterField(
            model_name='session',
            name='shopper_locale',
            field=models.CharField(blank=True, choices=[('fr_BE', 'French (Belgium)'), ('en_GB', 'English (United Kingdom)'), ('de_DE', 'German (Germany)'), ('nl_NL', 'Dutch (Holland)'), ('nl_BE', 'Dutch (Belgium)')], default='nl_NL', max_length=5),
        ),
        migrations.AlterField(
            model_name='sessionallowedpaymentmethods',
            name='method',
            field=models.CharField(choices=[('paypal', 'PayPal'), ('bankTransfer_DE', 'German Banktransfer'), ('card', 'All debit and credit cards'), ('visa', 'Visa'), ('bankTransfer', 'All banktransfers'), ('elv', 'ELV'), ('amex', 'Amex'), ('mc', 'Master Card'), ('directdebit_NL', 'Direct Debit (Netherlands)'), ('ideal', 'iDEAL'), ('directEbanking', 'SofortUberweisung'), ('bankTransfer_NL', 'Dutch Banktransfer')], max_length=50),
        ),
        migrations.AlterField(
            model_name='sessionblockedpaymentmethods',
            name='method',
            field=models.CharField(choices=[('paypal', 'PayPal'), ('bankTransfer_DE', 'German Banktransfer'), ('card', 'All debit and credit cards'), ('visa', 'Visa'), ('bankTransfer', 'All banktransfers'), ('elv', 'ELV'), ('amex', 'Amex'), ('mc', 'Master Card'), ('directdebit_NL', 'Direct Debit (Netherlands)'), ('ideal', 'iDEAL'), ('directEbanking', 'SofortUberweisung'), ('bankTransfer_NL', 'Dutch Banktransfer')], max_length=50),
        ),
    ]



import json
import textwrap
import requests
from . import settings


def exec_recurring_payment(
    contract_type,
    shopper_interaction,
    shopper_reference,
    shopper_email,
    merchant_reference,
    payment_amount,
    currency_code
):
    return payment_api_request(
        'authorise',
        {
            'amount': {
                'value': int(payment_amount),
                'currency': currency_code
            },
            'reference': merchant_reference,
            'merchantAccount': settings.MERCHANT_ACCOUNT,
            'shopperEmail': shopper_email,
            'shopperReference': shopper_reference,
            'selectedRecurringDetailReference': 'LATEST',
            'shopperInteraction': shopper_interaction,
            'recurring': {
                'contract': contract_type
            }
        }
    )


def list_recurring_details(shopper_reference, contract_type):
    return recurring_api_request(
        'listRecurringDetails',
        {
            'merchantAccount': settings.MERCHANT_ACCOUNT,
            'recurring': {
                'contract': contract_type
            },
            'shopperReference': shopper_reference
        }
    )


def disable_recurring_details(shopper_reference, recurring_detail_reference):
    return recurring_api_request(
        'disable',
        {
            'merchantAccount': settings.MERCHANT_ACCOUNT,
            'shopperReference': shopper_reference,
            'recurringDetailReference': recurring_detail_reference
        }
    )


def payment_api_request(endpoint, data):
    return api_request(
        '{}{}'.format(settings.PAYMENT_API_BASE_URL, endpoint),
        data
    )


def recurring_api_request(endpoint, data):
    return api_request(
        '{}{}'.format(settings.RECURRING_API_BASE_URL, endpoint),
        data
    )


class APIResponseInvalidJson(Exception):
    pass


def api_request(url, data):

    response = requests.post(
        url,
        headers={'Content-Type': 'application/json'},
        auth=(settings.API_USERNAME, settings.API_PASSWORD),
        data=json.dumps(data)
    )

    try:
        return response.json()
    except json.JSONDecodeError:
        raise APIResponseInvalidJson(
            "The response from the Adyen API was invalid JSON:\n"
            "  Status code: {}\n"
            "  Body: {}".format(
                response.status_code,
                textwrap.shorten(response.text, width=75)
            )
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-03 12:40
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('adyengo', '0018_auto_20161013_1848'),
    ]

    operations = [
        migrations.AlterField(
            model_name='recurringpaymentresult',
            name='psp_reference',
            field=models.BigIntegerField(),
        ),
    ]



from django.core import urlresolvers
from django.contrib import admin
from .models import (
    Session, SessionAllowedPaymentMethods, SessionBlockedPaymentMethods,
    Notification, RecurringContract, RecurringContractDetail,
    RecurringPaymentResult
)


class SessionAllowedPaymentMethodsInline(admin.TabularInline):
    model = SessionAllowedPaymentMethods
    extra = 0


class SessionBlockedPaymentMethodsInline(admin.TabularInline):
    model = SessionBlockedPaymentMethods
    extra = 0


class NotificationInline(admin.TabularInline):
    model = Notification
    extra = 0


class RecurringPaymentResultInline(admin.TabularInline):
    model = RecurringPaymentResult
    extra = 0


class SessionAdmin(admin.ModelAdmin):

    list_display = [
        'session_type', 'page_type', 'merchant_reference', 'payment_amount',
        'currency_code', 'shopper_locale', 'shopper_reference',
        'recurring_contract', 'creation_time'
    ]

    readonly_fields = ['creation_time']

    inlines = [
        SessionAllowedPaymentMethodsInline,
        SessionBlockedPaymentMethodsInline,
        NotificationInline,
        RecurringPaymentResultInline
    ]


class NotificationAdmin(admin.ModelAdmin):

    list_display = [
        'event_code', 'psp_reference', 'live', 'merchant_account_code',
        'payment_amount', 'currency_code', 'success', 'event_date', 'session',
        'creation_time'
    ]
    raw_id_fields = ['session']
    readonly_fields = ['creation_time', 'session_link']

    def session_link(self, instance):

        if instance.session.id:
            change_url = urlresolvers.reverse(
                'admin:adyengo_session_change',
                args=(instance.session.id,)
            )

            return '<a class="changelink" href="{}">Session</a>'.format(change_url)

        else:
            return 'No related session found'

    session_link.allow_tags = True


class RecurringContractDetailInline(admin.TabularInline):
    model = RecurringContractDetail
    extra = 0


class RecurringContractAdmin(admin.ModelAdmin):
    list_display = [
        'recurring_detail_reference', 'shopper_reference', 'contract_type',
        'payment_method_type', 'variant', 'creation_date'
    ]
    readonly_fields = ['creation_time']
    inlines = (RecurringContractDetailInline,)


class RecurringPaymentResultAdmin(admin.ModelAdmin):

    list_display = ['session', 'psp_reference', 'result_code', 'auth_code']
    raw_id_fields = ['session']
    readonly_fields = ['session_link']

    def session_link(self, instance):

        if instance.session.id:
            change_url = urlresolvers.reverse(
                'admin:adyengo_session_change', args=(instance.session.id,)
            )

            return '<a class="changelink" href="{}">Session</a>'.format(change_url)

        else:
            return 'No related session found'

    session_link.allow_tags = True


admin.site.register(Session, SessionAdmin)
admin.site.register(Notification, NotificationAdmin)
admin.site.register(RecurringContract, RecurringContractAdmin)
admin.site.register(RecurringPaymentResult, RecurringPaymentResultAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-03-04 14:58
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ip_address', models.CharField(max_length=45)),
                ('live', models.BooleanField()),
                ('event_code', models.CharField(blank=True, max_length=50)),
                ('psp_reference', models.CharField(blank=True, max_length=150)),
                ('original_reference', models.CharField(blank=True, max_length=150)),
                ('merchant_reference', models.CharField(blank=True, max_length=128)),
                ('merchant_account_code', models.CharField(blank=True, max_length=150)),
                ('event_date', models.DateTimeField(blank=True)),
                ('success', models.BooleanField()),
                ('payment_method', models.CharField(blank=True, max_length=50)),
                ('operations', models.CharField(blank=True, max_length=100)),
                ('reason', models.CharField(blank=True, max_length=250)),
                ('amount', models.CharField(blank=True, max_length=15)),
                ('valid', models.NullBooleanField()),
                ('creation_time', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ('-creation_time',),
            },
        ),
        migrations.CreateModel(
            name='RecurringContract',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('recurring_detail_reference', models.CharField(blank=True, max_length=150)),
                ('shopper_reference', models.CharField(max_length=80)),
                ('contract_type', models.CharField(max_length=50)),
                ('payment_method_type', models.CharField(max_length=15)),
                ('variant', models.CharField(blank=True, max_length=50)),
                ('creation_date', models.DateTimeField(blank=True)),
                ('creation_time', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ('-creation_time',),
            },
        ),
        migrations.CreateModel(
            name='RecurringContractDetail',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=100)),
                ('value', models.CharField(blank=True, max_length=250)),
                ('recurring_contract', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='details', to='adyengo.RecurringContract')),
            ],
        ),
        migrations.CreateModel(
            name='RecurringPaymentResult',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('psp_reference', models.PositiveIntegerField()),
                ('result_code', models.CharField(choices=[('Authorised', 'Authorised'), ('Refused', 'Refused'), ('Error', 'Error')], max_length=30)),
                ('auth_code', models.PositiveIntegerField(null=True)),
                ('refusal_reason', models.CharField(blank=True, max_length=250)),
            ],
        ),
        migrations.CreateModel(
            name='Session',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('session_type', models.CharField(choices=[('hpp_regular', 'HPP Regular'), ('hpp_recurring', 'HPP Recurring'), ('api_recurring', 'API Recurring')], max_length=25)),
                ('merchant_reference', models.CharField(max_length=80, unique=True)),
                ('payment_amount', models.PositiveSmallIntegerField()),
                ('currency_code', models.CharField(choices=[('EUR', 'Euro')], default=None, max_length=3)),
                ('ship_before_date', models.DateTimeField(null=True)),
                ('skin_code', models.CharField(default='BCS1MHG2', max_length=10)),
                ('shopper_locale', models.CharField(blank=True, choices=[('en_GB', 'English (United Kingdom)'), ('nl_BE', 'Dutch (Belgium)'), ('nl_NL', 'Dutch (Holland)'), ('fr_BE', 'French (Belgium)'), ('de_DE', 'German (Germany)')], max_length=5)),
                ('order_data', models.TextField(blank=True)),
                ('session_validity', models.DateTimeField(null=True)),
                ('merchant_return_data', models.CharField(max_length=128)),
                ('country_code', models.CharField(blank=True, choices=[('BE', 'Belgium'), ('NL', 'Netherlands'), ('DE', 'Germany'), ('GB', 'United Kingdom')], max_length=2)),
                ('shopper_email', models.EmailField(blank=True, max_length=254)),
                ('shopper_reference', models.CharField(blank=True, max_length=80)),
                ('shopper_ip', models.CharField(blank=True, max_length=45)),
                ('shopper_statement', models.CharField(blank=True, max_length=135)),
                ('fraud_offset', models.PositiveIntegerField(null=True)),
                ('recurring_contract', models.CharField(blank=True, choices=[('RECURRING', 'Recurring'), ('RECURRING,ONECLICK', 'Recurring and One click (user chooses)'), ('ONECLICK', 'One click')], max_length=50)),
                ('recurring_detail_reference', models.CharField(blank=True, max_length=80)),
                ('page_type', models.CharField(choices=[('skip', 'Skip'), ('single', 'Single'), ('multiple', 'Multiple')], max_length=15)),
                ('creation_time', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ('-creation_time',),
            },
        ),
        migrations.CreateModel(
            name='SessionAllowedPaymentMethods',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('method', models.CharField(choices=[('bankTransfer', 'All banktransfers'), ('paypal', 'PayPal'), ('ideal', 'iDEAL'), ('bankTransfer_DE', 'German Banktransfer'), ('bankTransfer_NL', 'Dutch Banktransfer'), ('card', 'All debit and credit cards'), ('elv', 'ELV'), ('mc', 'Master Card'), ('visa', 'Visa'), ('directdebit_NL', 'Direct Debit (Netherlands)'), ('amex', 'Amex'), ('directEbanking', 'SofortUberweisung')], max_length=50)),
                ('session', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='allowed_payment_methods', to='adyengo.Session')),
            ],
        ),
        migrations.CreateModel(
            name='SessionBlockedPaymentMethods',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('method', models.CharField(choices=[('bankTransfer', 'All banktransfers'), ('paypal', 'PayPal'), ('ideal', 'iDEAL'), ('bankTransfer_DE', 'German Banktransfer'), ('bankTransfer_NL', 'Dutch Banktransfer'), ('card', 'All debit and credit cards'), ('elv', 'ELV'), ('mc', 'Master Card'), ('visa', 'Visa'), ('directdebit_NL', 'Direct Debit (Netherlands)'), ('amex', 'Amex'), ('directEbanking', 'SofortUberweisung')], max_length=50)),
                ('session', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='blocked_payment_methods', to='adyengo.Session')),
            ],
        ),
        migrations.AddField(
            model_name='recurringpaymentresult',
            name='session',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='recurring_payment_results', to='adyengo.Session'),
        ),
        migrations.AlterUniqueTogether(
            name='recurringcontract',
            unique_together=set([('recurring_detail_reference', 'shopper_reference')]),
        ),
        migrations.AddField(
            model_name='notification',
            name='session',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='adyengo.Session'),
        ),
        migrations.AlterUniqueTogether(
            name='notification',
            unique_together=set([('live', 'merchant_account_code', 'psp_reference', 'event_code', 'success')]),
        ),
    ]



from django.db import models
from . import api
from . import constants


class RecurringContractManager(models.Manager):

    def contracts(self, shopper_reference, contract_type):
        """
        Returns a list of Recurring Payment Contracts.

        It firsts looks in the "cache" (database), if it can't find any
        records, it will use `_fetch_contracts()` which will fetch the
        contracts from the Adyen API.
        """

        contracts = self.filter(
            shopper_reference=shopper_reference,
            contract_type=contract_type
        )

        if contracts.count():
            return contracts
        else:
            return self._fetch_contracts(shopper_reference, contract_type)

    def disable(self, shopper_reference, recurring_detail_reference):

        response = api.disable_recurring_details(shopper_reference, recurring_detail_reference)

        if response.get('response') == '[detail-successfully-disabled]':
            self.filter(
                shopper_reference=shopper_reference,
                recurring_detail_reference=recurring_detail_reference
            ).delete()
            return True

    def flush_cache(self, shopper_reference, contract_type):
        """
        Flushes the Recurring Payment Contracts cache.

        This will ensure that the next time `contracts()` is executed, it won't
        find any contracts in the cache and will do a request to the Adyen API
        (and will refill the cache).
        """
        self.filter(
            shopper_reference=shopper_reference,
            contract_type=contract_type
        ).delete()

    def _fetch_contracts(self, shopper_reference, contract_type):
        """
        Will fetch the Recurring Payment Contracts from the Adyen API, save
        them to the database and return the list of `RecurringContract` models.

        If there are no Recurring Payment Contracts, it won't save anything to
        the database and will return an empty list.
        """

        result = api.list_recurring_details(shopper_reference, contract_type)

        if 'details' in result:
            return self._save_contracts(shopper_reference, contract_type, result)
        else:
            return []

    def _save_contracts(self, shopper_reference, contract_type, result):
        """
        Saves the contracts returned from the Adyen API to the database and
        returns the list of `RecurringContract` models.
        """

        contracts = []

        for contract in result.get('details'):

            recurring_detail = contract.get('RecurringDetail')

            if not recurring_detail:
                continue

            contracts.append(
                self._save_contract(
                    shopper_reference,
                    contract_type,
                    recurring_detail
                )
            )

        return contracts

    def _save_contract(self, shopper_reference, contract_type, data):

        def get_payment_method_type():
            # The API is too retarded to tell us what payment method type
            # it is so we'll have to find out for ourselves by checking
            # which one is filled...
            for t in ('card', 'elv', 'bank'):
                if t in data:
                    return t

        payment_method_type = get_payment_method_type()

        contract, created = self.get_or_create(
            shopper_reference=shopper_reference,
            contract_type=contract_type,
            recurring_detail_reference=data.get('recurringDetailReference'),
            variant=data.get('variant'),
            payment_method_type=payment_method_type,
            creation_date=data.get('creationDate')
        )

        # Each payment method type has different fields, we save these in
        # the `RecurringContractDetail` objects that have foreign keys
        # to `RecurringContract`.

        fields = data[payment_method_type]

        for fieldname in constants.RECURRING_CONTRACT_VARIANT_FIELDS[payment_method_type]:

            if fieldname in fields:
                contract.details.create(
                    recurring_contract=contract,
                    key=fieldname,
                    value=fields[fieldname]
                )

        return contract



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-16 11:29
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('adyengo', '0019_auto_20170303_1340'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='payment_method',
            field=models.CharField(blank=True, choices=[('amex', 'Amex'), ('bankTransfer', 'All banktransfers'), ('bankTransfer_DE', 'German Banktransfer'), ('bankTransfer_IBAN', 'International Bank Transfer (IBAN)'), ('bankTransfer_NL', 'Dutch Banktransfer'), ('bcmc', 'Bancontact card'), ('card', 'All debit and credit cards'), ('diners', 'Diners Club'), ('directEbanking', 'SofortUberweisung'), ('directdebit_NL', 'Direct Debit (Netherlands)'), ('discover', 'Discover'), ('dotpay', 'Dotpay'), ('ebanking_FI', 'Finnish E-Banking'), ('elv', 'ELV'), ('giropay', 'GiroPay'), ('ideal', 'iDEAL'), ('maestro', 'Maestro'), ('mc', 'Master Card'), ('paypal', 'PayPal'), ('sepadirectdebit', 'SEPA Direct Debit'), ('visa', 'Visa')], max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='recurringpaymentresult',
            name='result_code',
            field=models.CharField(choices=[('Authorised', 'Authorised'), ('Error', 'Error'), ('Received', 'Received'), ('Refused', 'Refused')], max_length=30),
        ),
        migrations.AlterField(
            model_name='session',
            name='country_code',
            field=models.CharField(choices=[('BE', 'Belgium'), ('DE', 'Germany'), ('GB', 'United Kingdom'), ('NL', 'Netherlands')], max_length=2),
        ),
        migrations.AlterField(
            model_name='session',
            name='page_type',
            field=models.CharField(choices=[('multiple', 'Multiple'), ('single', 'Single'), ('skip', 'Skip')], default='skip', max_length=15),
        ),
        migrations.AlterField(
            model_name='session',
            name='recurring_contract',
            field=models.CharField(blank=True, choices=[('ONECLICK', 'One click'), ('RECURRING', 'Recurring'), ('RECURRING,ONECLICK', 'Recurring and One click (user chooses)')], max_length=50),
        ),
        migrations.AlterField(
            model_name='session',
            name='shopper_locale',
            field=models.CharField(blank=True, choices=[('de_DE', 'German (Germany)'), ('en_GB', 'English (United Kingdom)'), ('fr_BE', 'French (Belgium)'), ('nl_BE', 'Dutch (Belgium)'), ('nl_NL', 'Dutch (Holland)')], default='nl_NL', max_length=5),
        ),
        migrations.AlterField(
            model_name='sessionallowedpaymentmethods',
            name='method',
            field=models.CharField(choices=[('amex', 'Amex'), ('bankTransfer', 'All banktransfers'), ('bankTransfer_DE', 'German Banktransfer'), ('bankTransfer_IBAN', 'International Bank Transfer (IBAN)'), ('bankTransfer_NL', 'Dutch Banktransfer'), ('bcmc', 'Bancontact card'), ('card', 'All debit and credit cards'), ('diners', 'Diners Club'), ('directEbanking', 'SofortUberweisung'), ('directdebit_NL', 'Direct Debit (Netherlands)'), ('discover', 'Discover'), ('dotpay', 'Dotpay'), ('ebanking_FI', 'Finnish E-Banking'), ('elv', 'ELV'), ('giropay', 'GiroPay'), ('ideal', 'iDEAL'), ('maestro', 'Maestro'), ('mc', 'Master Card'), ('paypal', 'PayPal'), ('sepadirectdebit', 'SEPA Direct Debit'), ('visa', 'Visa')], max_length=50),
        ),
        migrations.AlterField(
            model_name='sessionblockedpaymentmethods',
            name='method',
            field=models.CharField(choices=[('amex', 'Amex'), ('bankTransfer', 'All banktransfers'), ('bankTransfer_DE', 'German Banktransfer'), ('bankTransfer_IBAN', 'International Bank Transfer (IBAN)'), ('bankTransfer_NL', 'Dutch Banktransfer'), ('bcmc', 'Bancontact card'), ('card', 'All debit and credit cards'), ('diners', 'Diners Club'), ('directEbanking', 'SofortUberweisung'), ('directdebit_NL', 'Direct Debit (Netherlands)'), ('discover', 'Discover'), ('dotpay', 'Dotpay'), ('ebanking_FI', 'Finnish E-Banking'), ('elv', 'ELV'), ('giropay', 'GiroPay'), ('ideal', 'iDEAL'), ('maestro', 'Maestro'), ('mc', 'Master Card'), ('paypal', 'PayPal'), ('sepadirectdebit', 'SEPA Direct Debit'), ('visa', 'Visa')], max_length=50),
        ),
    ]



from setuptools import setup, find_packages


setup(
    name='adyengo',
    keywords='django adyen payment integration',
    version='0.0.10',
    author='gitaarik',
    author_email='gitaarik@gmail.com',
    description='Django app for easy Adyen integration.',
    url='https://github.com/gitaarik/adyengo',
    license='LGPL licence, see LICENCE.txt',
    packages=find_packages(),
    include_package_data=True,
    install_requires=('ipaddress', 'python-dateutil', 'requests'),
    zip_safe=False,
)



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-07-07 17:19
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('adyengo', '0012_auto_20160610_1504'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='payment_method',
            field=models.CharField(blank=True, choices=[('directdebit_NL', 'Direct Debit (Netherlands)'), ('amex', 'Amex'), ('elv', 'ELV'), ('ideal', 'iDEAL'), ('card', 'All debit and credit cards'), ('bankTransfer_DE', 'German Banktransfer'), ('directEbanking', 'SofortUberweisung'), ('paypal', 'PayPal'), ('mc', 'Master Card'), ('visa', 'Visa'), ('bankTransfer', 'All banktransfers'), ('bankTransfer_NL', 'Dutch Banktransfer')], max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='recurringpaymentresult',
            name='result_code',
            field=models.CharField(choices=[('Received', 'Received'), ('Refused', 'Refused'), ('Error', 'Error'), ('Authorised', 'Authorised')], max_length=30),
        ),
        migrations.AlterField(
            model_name='session',
            name='country_code',
            field=models.CharField(choices=[('BE', 'Belgium'), ('DE', 'Germany'), ('GB', 'United Kingdom'), ('NL', 'Netherlands')], max_length=2),
        ),
        migrations.AlterField(
            model_name='session',
            name='page_type',
            field=models.CharField(choices=[('single', 'Single'), ('skip', 'Skip'), ('multiple', 'Multiple')], default='multiple', max_length=15),
        ),
        migrations.AlterField(
            model_name='session',
            name='session_type',
            field=models.CharField(choices=[('hpp_regular', 'HPP Regular'), ('api_recurring', 'API Recurring'), ('hpp_recurring', 'HPP Recurring')], max_length=25),
        ),
        migrations.AlterField(
            model_name='session',
            name='shopper_locale',
            field=models.CharField(blank=True, choices=[('fr_BE', 'French (Belgium)'), ('de_DE', 'German (Germany)'), ('en_GB', 'English (United Kingdom)'), ('nl_BE', 'Dutch (Belgium)'), ('nl_NL', 'Dutch (Holland)')], default='nl_NL', max_length=5),
        ),
        migrations.AlterField(
            model_name='sessionallowedpaymentmethods',
            name='method',
            field=models.CharField(choices=[('directdebit_NL', 'Direct Debit (Netherlands)'), ('amex', 'Amex'), ('elv', 'ELV'), ('ideal', 'iDEAL'), ('card', 'All debit and credit cards'), ('bankTransfer_DE', 'German Banktransfer'), ('directEbanking', 'SofortUberweisung'), ('paypal', 'PayPal'), ('mc', 'Master Card'), ('visa', 'Visa'), ('bankTransfer', 'All banktransfers'), ('bankTransfer_NL', 'Dutch Banktransfer')], max_length=50),
        ),
        migrations.AlterField(
            model_name='sessionblockedpaymentmethods',
            name='method',
            field=models.CharField(choices=[('directdebit_NL', 'Direct Debit (Netherlands)'), ('amex', 'Amex'), ('elv', 'ELV'), ('ideal', 'iDEAL'), ('card', 'All debit and credit cards'), ('bankTransfer_DE', 'German Banktransfer'), ('directEbanking', 'SofortUberweisung'), ('paypal', 'PayPal'), ('mc', 'Master Card'), ('visa', 'Visa'), ('bankTransfer', 'All banktransfers'), ('bankTransfer_NL', 'Dutch Banktransfer')], max_length=50),
        ),
    ]



import base64
import binascii
import hashlib
import hmac
import ipaddress
from collections import OrderedDict
from . import settings, constants


def is_string(var):
    try:
        return isinstance(var, basestring)
    except NameError:
        return isinstance(var, str)


def remove_items_with_empty_values(params):
    return {
        key: value
        for key, value in params.items()
        if value
    }


def merchant_sig(params, clean_params=True):

    def clean_params(params):

        valid_params = (
            'allowedMethods', 'authResult', 'blockedMethods', 'brandCode',
            'countryCode', 'currencyCode', 'currencyCode', 'merchantAccount',
            'merchantReference', 'merchantReturnData', 'orderData',
            'paymentAmount', 'paymentMethod', 'pspReference',
            'recurringContract', 'resURL', 'selectedRecurringDetailReference',
            'sessionValidity', 'shipBeforeDate', 'shopperEmail',
            'shopperLocale', 'shopperReference', 'skinCode'
        )

        return {
            key: value
            for key, value in params.items()
            if key in valid_params
        }

    if clean_params:
        params = clean_params(params)

    params = OrderedDict(sorted(params.items()))

    assert is_string(settings.HMAC_KEY), (
        "Invalid `HMAC_KEY`, did you set the `HMAC_KEY` in the Adyengo settings dict?"
    )

    return calc_hmac(
        ':'.join(
            list(params.keys()) +
            escape_values(params.values())
        ),
        settings.HMAC_KEY
    )


def calc_hmac(string, hmac_key):

    return base64.encodestring(hmac.new(
        binascii.a2b_hex(hmac_key.encode()),
        string.encode(),
        hashlib.sha256
    ).digest()).strip().decode()


def escape_values(values):
    return [
        str(value).replace(r'\\', r'\\\\').replace(r':', r'\:')
        for value in values
    ]


def is_notification_item_hmac_valid(item):

    hmac_values = (
        item.get('pspReference', ''),
        item.get('originalReference', ''),
        item.get('merchantAccountCode', ''),
        item.get('merchantReference', ''),
        item.get('amount', {}).get('value', ''),
        item.get('amount', {}).get('currency', ''),
        item.get('eventCode'),
        item.get('success'),
    )

    hmac_signature = item.get('additionalData', {}).get('hmacSignature')

    assert is_string(settings.NOTIFICATION_HMAC_KEY), (
        "Invalid `NOTIFICATION_HMAC_KEY`, did you set the "
        "`NOTIFICATION_HMAC_KEY` in the Adyengo settings dict?"
    )

    if hmac_signature and hmac_signature == calc_hmac(
        ':'.join(escape_values(hmac_values)),
        settings.NOTIFICATION_HMAC_KEY
    ):
        return True


def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    if isinstance(ip, bytes):
        return ip.decode('utf-8')
    return ip


def is_valid_ip(ip_address):
    for allowed_ip in constants.ADYEN_SERVERS_IP_ADDRESS_RANGES:
        if ipaddress.ip_address(ip_address) in ipaddress.ip_network(allowed_ip):
            return True
